<!--
~ Copyright (c) 2021, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
~
~ This software is the property of WSO2 Inc. and its suppliers, if any.
~ Dissemination of any information or reproduction of any material contained
~ herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
~ You may not alter or remove any copyright or other notice from copies of this content."
-->

<!-- init-url.jsp must be included in the calling script inorder to resolve tenant context -->

<%@ page import="org.json.JSONObject" %>
<%@ page import="org.apache.commons.lang.StringUtils" %>
<%@ page import="org.wso2.carbon.identity.mgt.endpoint.util.client.BrandingPreferenceRetrievalClient" %>
<%@ page import="org.wso2.carbon.identity.mgt.endpoint.util.client.BrandingPreferenceRetrievalClientException" %>
<%@ page import="org.wso2.carbon.identity.application.authentication.endpoint.util.AuthenticationEndpointUtil" %>
<%@ page import="org.wso2.carbon.identity.mgt.endpoint.util.IdentityManagementEndpointConstants" %>

<%
    JSONObject brandingPreference = null;

    boolean isBrandingEnabledInTenantPreferences = true;
    boolean isSelfSignUpEnabledInTenantPreferences = true;
    boolean shouldRemoveDefaultBranding = false;
    String overrideStylesheet = "";
    String copyrightText = "";
    String siteTitle = "Asgardeo by WSO2";
    String supportEmail = "asgardeo-help@wso2.com";
    String logoURL = "libs/themes/asgardio/assets/images/branding/asgardeo-logo.svg";
    String logoAlt = "";
    String faviconURL = "libs/themes/asgardio/assets/images/branding/favicon.ico";
    String privacyPolicyURL = application.getInitParameter("PrivacyPolicyURL");
    String termsOfUseURL = application.getInitParameter("TermsOfServiceURL");
    String cookiePolicyURL = "https://wso2.com/cookie-policy";
    String selfSignUpOverrideURL = "";

    // If `PrivacyPolicyURL` is not configured in the configuration (web.xml) fallback to default.
    if (StringUtils.isBlank(termsOfUseURL)) {
        termsOfUseURL = "https://wso2.com/asgardeo/terms-of-use";
    }

    // If `PrivacyPolicyURL` is not configured in the configuration (web.xml) fallback to default.
    if (StringUtils.isBlank(privacyPolicyURL)) {
        privacyPolicyURL = "https://wso2.com/asgardeo/privacy-policy";
    }

    try {
        String DEFAULT_RESOURCE_LOCALE = "en-US";
        String ORG_PREFERENCE_RESOURCE_TYPE = "ORG";
        String APP_PREFERENCE_RESOURCE_TYPE = "APP";

        String tenantRequestingPreferences = tenantForTheming;
        String applicationRequestingPreferences = spAppName;
        String preferenceResourceType = ORG_PREFERENCE_RESOURCE_TYPE;

        // If the `sp` param is defined, set the resource type as app.
        if (StringUtils.isNotBlank(applicationRequestingPreferences)) {
            preferenceResourceType = APP_PREFERENCE_RESOURCE_TYPE;
        }

        // Only engage the preference retrieval if the flow is not super tenant.
        if (!StringUtils.equals(tenantRequestingPreferences, IdentityManagementEndpointConstants.SUPER_TENANT)) {

            BrandingPreferenceRetrievalClient brandingPreferenceRetrievalClient = new BrandingPreferenceRetrievalClient();
            JSONObject brandingPreferenceResponse = brandingPreferenceRetrievalClient.getPreference(tenantRequestingPreferences,
                    preferenceResourceType, applicationRequestingPreferences, DEFAULT_RESOURCE_LOCALE);

            // Preferences response object pointer keys.
            String PREFERENCE_KEY = "preference";
            String STYLESHEETS_KEY = "stylesheets";
            String ACCOUNT_APP_STYLESHEET_KEY = "accountApp";
            String ORG_DETAILS_KEY = "organizationDetails";
            String COPYRIGHT_TEXT_KEY = "copyrightText";
            String SITE_TITLE_KEY = "siteTitle";
            String SUPPORT_EMAIL_KEY = "supportEmail";
            String IMAGES_KEY = "images";
            String IMAGE_URL_KEY = "imgURL";
            String ALT_TEXT_KEY = "altText";
            String LOGO_KEY = "logo";
            String FAVICON_KEY = "favicon";
            String URLS_KEY = "urls";
            String PRIVACY_POLICY_URL_KEY = "privacyPolicyURL";
            String TERMS_OF_USE_URL_KEY = "termsOfUseURL";
            String COOKIE_POLICY_URL_KEY = "cookiePolicyURL";
            String SELF_SIGN_UP_URL_KEY = "selfSignUpURL";
            String CONFIGS_KEY = "configs";
            String IS_BRANDING_ENABLED_KEY= "isBrandingEnabled";
            String IS_SELF_SIGN_UP_ENABLED_KEY = "isSelfSignUpEnabled";
            String SHOULD_REMOVE_DEFAULT_BRANDING_KEY = "removeDefaultBranding";

            if (brandingPreferenceResponse.has(PREFERENCE_KEY)) {
                brandingPreference = brandingPreferenceResponse.getJSONObject(PREFERENCE_KEY);
            }

            // Proceed only if `preferences` object is defined.
            if (brandingPreference != null) {
                // Configs
                if (brandingPreference.has(CONFIGS_KEY)) {
                    if (brandingPreference.getJSONObject(CONFIGS_KEY).has(IS_BRANDING_ENABLED_KEY)) {
                        isBrandingEnabledInTenantPreferences = brandingPreference.getJSONObject(CONFIGS_KEY).getBoolean(IS_BRANDING_ENABLED_KEY);
                    }

                    if (brandingPreference.getJSONObject(CONFIGS_KEY).has(IS_SELF_SIGN_UP_ENABLED_KEY)) {
                        isSelfSignUpEnabledInTenantPreferences = brandingPreference.getJSONObject(CONFIGS_KEY).getBoolean(IS_SELF_SIGN_UP_ENABLED_KEY);
                    }

                    // TODO: Need to integrate this config in to the code.
                    // First should identify the places where branding should be removed based on the subscription model.
                    // Tracked here: https://github.com/wso2-enterprise/asgardeo-product/issues/8357
                    if (brandingPreference.getJSONObject(CONFIGS_KEY).has(SHOULD_REMOVE_DEFAULT_BRANDING_KEY)) {
                        shouldRemoveDefaultBranding = brandingPreference.getJSONObject(CONFIGS_KEY).getBoolean(SHOULD_REMOVE_DEFAULT_BRANDING_KEY);
                    }
                }

                // Proceed only if the branding is enabled.
                if (isBrandingEnabledInTenantPreferences) {
                    // Stylesheets
                    if (brandingPreference.has(STYLESHEETS_KEY)) {
                        overrideStylesheet = brandingPreference.getJSONObject(STYLESHEETS_KEY).getString(ACCOUNT_APP_STYLESHEET_KEY);
                    }

                    // Organization Details
                    if (brandingPreference.has(ORG_DETAILS_KEY)) {
                        if (brandingPreference.getJSONObject(ORG_DETAILS_KEY).has(COPYRIGHT_TEXT_KEY)) {
                            copyrightText = brandingPreference.getJSONObject(ORG_DETAILS_KEY).getString(COPYRIGHT_TEXT_KEY);
                        }

                        if (brandingPreference.getJSONObject(ORG_DETAILS_KEY).has(SITE_TITLE_KEY)) {
                            siteTitle = brandingPreference.getJSONObject(ORG_DETAILS_KEY).getString(SITE_TITLE_KEY);
                        }

                        if (brandingPreference.getJSONObject(ORG_DETAILS_KEY).has(SUPPORT_EMAIL_KEY)) {
                            supportEmail = brandingPreference.getJSONObject(ORG_DETAILS_KEY).getString(SUPPORT_EMAIL_KEY);
                        }
                    }

                    // Images
                    if (brandingPreference.has(IMAGES_KEY)) {
                        if (brandingPreference.getJSONObject(IMAGES_KEY).has(LOGO_KEY)) {
                            if (brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(LOGO_KEY).has(IMAGE_URL_KEY)) {
                                logoURL = brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(LOGO_KEY).getString(IMAGE_URL_KEY);
                            }

                            if (brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(LOGO_KEY).has(IMAGE_URL_KEY)) {
                                logoAlt = brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(LOGO_KEY).getString(ALT_TEXT_KEY);
                            }
                        }

                        if (brandingPreference.getJSONObject(IMAGES_KEY).has(FAVICON_KEY)) {
                            if (brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(FAVICON_KEY).has(IMAGE_URL_KEY)) {
                                faviconURL = brandingPreference.getJSONObject(IMAGES_KEY).getJSONObject(FAVICON_KEY).getString(IMAGE_URL_KEY);
                            }
                        }
                    }

                    // Links
                    if (brandingPreference.has(URLS_KEY)) {
                        if (brandingPreference.getJSONObject(URLS_KEY).has(PRIVACY_POLICY_URL_KEY)) {
                            privacyPolicyURL = brandingPreference.getJSONObject(URLS_KEY).getString(PRIVACY_POLICY_URL_KEY);
                        }

                        if (brandingPreference.getJSONObject(URLS_KEY).has(TERMS_OF_USE_URL_KEY)) {
                            termsOfUseURL = brandingPreference.getJSONObject(URLS_KEY).getString(TERMS_OF_USE_URL_KEY);
                        }

                        if (brandingPreference.getJSONObject(URLS_KEY).has(COOKIE_POLICY_URL_KEY)) {
                            cookiePolicyURL = brandingPreference.getJSONObject(URLS_KEY).getString(COOKIE_POLICY_URL_KEY);
                        }

                        if (brandingPreference.getJSONObject(URLS_KEY).has(SELF_SIGN_UP_URL_KEY)) {
                            selfSignUpOverrideURL = brandingPreference.getJSONObject(URLS_KEY).getString(SELF_SIGN_UP_URL_KEY);
                        }
                    }
                }
            }
        }
    } catch (BrandingPreferenceRetrievalClientException e) {
        // Exception is ignored and the variable will use the fallbacks.
    }
%>
